import * as program from 'commander'
import { prompt } from 'inquirer'
import { sessionManager, UcRegistrarPonto } from '../../curio'
import { Session, SystemError, BusinessError, ConnectionError } from 'curiojs'
import * as questions from '../../constants/questions'
import * as chalk from 'chalk'
import * as makeSpinner from 'ora'
import entrada from './entrada'
import saida from './saida'

program
  .command('ponto')
  .usage('[opções]')
  .option('-u, --username <username>', 'Adicione o nome do usuário')
  .option('-p, --password <password>', 'Adicione o password diretamente na linha do comando')
  .action(async function ponto (options: program.CommanderStatic) {
    let {username, password} = options
    if (!username) username = (await prompt(questions.username)).username
    if (!password) password = (await prompt(questions.password)).password
    const spinner = makeSpinner()
    try {
      spinner.start(chalk.cyan('Realizando login...'))
      const session: Session<Promise<UcRegistrarPonto>> = await sessionManager.login(username, password)
      const ucRegistrarPonto = await session.open(UcRegistrarPonto)
      const {Domains} = await ucRegistrarPonto.obterContexto()
      const {UltimoRegistroPonto, RegistroPonto} = await ucRegistrarPonto.incluirRegistroPonto()
      spinner.succeed(`Login realizado com ${chalk.bold.italic.underline.green('sucesso')}`)
      if (UltimoRegistroPonto && UltimoRegistroPonto[0]._Tipo === 'E') saida(Domains[0], UltimoRegistroPonto[0], RegistroPonto[0], ucRegistrarPonto)
      else entrada(RegistroPonto[0], ucRegistrarPonto)
    } catch (error) {
      if (error instanceof BusinessError) spinner.fail(chalk.bgRed.white.bold(error.message))
      else if (error instanceof ConnectionError) spinner.fail(chalk.bgRed.white.bold(error.message))
      else if (error instanceof SystemError) spinner.fail(chalk.bgRed.white.bold(error.message))
      else console.error(error)
    }
  })
